@@ -147,7 +147,53 @@ glade_marshal_VOID__STRING_ULONG_UINT_STRING (GClosure     *closure,
 /* VOID:OBJECT (./glade-marshallers.list:4) */
 #define glade_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
 
-/* VOID:OBJECT,BOOLEAN (./glade-marshallers.list:5) */
+/* VOID:STRING (./glade-marshallers.list:5) */
+#define glade_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
+
+/* VOID:OBJECT,OBJECT (./glade-marshallers.list:6) */
+extern void glade_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
+                                               GValue       *return_value,
+                                               guint         n_param_values,
+                                               const GValue *param_values,
+                                               gpointer      invocation_hint,
+                                               gpointer      marshal_data);
+void
+glade_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
+                                   GValue       *return_value G_GNUC_UNUSED,
+                                   guint         n_param_values,
+                                   const GValue *param_values,
+                                   gpointer      invocation_hint G_GNUC_UNUSED,
+                                   gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__OBJECT_OBJECT) (gpointer     data1,
+                                                    gpointer     arg_1,
+                                                    gpointer     arg_2,
+                                                    gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_OBJECT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__OBJECT_OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_object (param_values + 1),
+            g_marshal_value_peek_object (param_values + 2),
+            data2);
+}
+
+/* VOID:OBJECT,BOOLEAN (./glade-marshallers.list:7) */
 extern void glade_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -190,7 +236,7 @@ glade_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,STRING (./glade-marshallers.list:6) */
+/* VOID:STRING,STRING,STRING (./glade-marshallers.list:8) */
 extern void glade_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -235,7 +281,7 @@ glade_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* OBJECT:POINTER (./glade-marshallers.list:7) */
+/* OBJECT:POINTER (./glade-marshallers.list:9) */
 extern void glade_marshal_OBJECT__POINTER (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -280,7 +326,7 @@ glade_marshal_OBJECT__POINTER (GClosure     *closure,
   g_value_take_object (return_value, v_return);
 }
 
-/* OBJECT:OBJECT,UINT (./glade-marshallers.list:8) */
+/* OBJECT:OBJECT,UINT (./glade-marshallers.list:10) */
 extern void glade_marshal_OBJECT__OBJECT_UINT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -327,7 +373,7 @@ glade_marshal_OBJECT__OBJECT_UINT (GClosure     *closure,
   g_value_take_object (return_value, v_return);
 }
 
-/* BOOLEAN:STRING (./glade-marshallers.list:9) */
+/* BOOLEAN:STRING (./glade-marshallers.list:11) */
 extern void glade_marshal_BOOLEAN__STRING (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -372,7 +418,7 @@ glade_marshal_BOOLEAN__STRING (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:BOXED (./glade-marshallers.list:10) */
+/* BOOLEAN:BOXED (./glade-marshallers.list:12) */
 extern void glade_marshal_BOOLEAN__BOXED (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -417,7 +463,7 @@ glade_marshal_BOOLEAN__BOXED (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:OBJECT (./glade-marshallers.list:11) */
+/* BOOLEAN:OBJECT (./glade-marshallers.list:13) */
 extern void glade_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -462,7 +508,7 @@ glade_marshal_BOOLEAN__OBJECT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:OBJECT,BOXED (./glade-marshallers.list:12) */
+/* BOOLEAN:OBJECT,BOXED (./glade-marshallers.list:14) */
 extern void glade_marshal_BOOLEAN__OBJECT_BOXED (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -509,7 +555,7 @@ glade_marshal_BOOLEAN__OBJECT_BOXED (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:OBJECT,POINTER (./glade-marshallers.list:13) */
+/* BOOLEAN:OBJECT,POINTER (./glade-marshallers.list:15) */
 extern void glade_marshal_BOOLEAN__OBJECT_POINTER (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -556,7 +602,7 @@ glade_marshal_BOOLEAN__OBJECT_POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:OBJECT,BOOLEAN (./glade-marshallers.list:14) */
+/* BOOLEAN:OBJECT,BOOLEAN (./glade-marshallers.list:16) */
 extern void glade_marshal_BOOLEAN__OBJECT_BOOLEAN (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -603,7 +649,7 @@ glade_marshal_BOOLEAN__OBJECT_BOOLEAN (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:OBJECT,UINT (./glade-marshallers.list:15) */
+/* BOOLEAN:OBJECT,UINT (./glade-marshallers.list:17) */
 extern void glade_marshal_BOOLEAN__OBJECT_UINT (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -650,7 +696,7 @@ glade_marshal_BOOLEAN__OBJECT_UINT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:OBJECT,OBJECT (./glade-marshallers.list:16) */
+/* BOOLEAN:OBJECT,OBJECT (./glade-marshallers.list:18) */
 extern void glade_marshal_BOOLEAN__OBJECT_OBJECT (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -697,7 +743,7 @@ glade_marshal_BOOLEAN__OBJECT_OBJECT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:OBJECT,STRING (./glade-marshallers.list:17) */
+/* BOOLEAN:OBJECT,STRING (./glade-marshallers.list:19) */
 extern void glade_marshal_BOOLEAN__OBJECT_STRING (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -744,7 +790,7 @@ glade_marshal_BOOLEAN__OBJECT_STRING (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:STRING,STRING,STRING,BOXED (./glade-marshallers.list:18) */
+/* BOOLEAN:STRING,STRING,STRING,BOXED (./glade-marshallers.list:20) */
 extern void glade_marshal_BOOLEAN__STRING_STRING_STRING_BOXED (GClosure     *closure,
                                                                GValue       *return_value,
                                                                guint         n_param_values,
@@ -795,7 +841,7 @@ glade_marshal_BOOLEAN__STRING_STRING_STRING_BOXED (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:STRING,BOXED,OBJECT (./glade-marshallers.list:19) */
+/* BOOLEAN:STRING,BOXED,OBJECT (./glade-marshallers.list:21) */
 extern void glade_marshal_BOOLEAN__STRING_BOXED_OBJECT (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -844,7 +890,7 @@ glade_marshal_BOOLEAN__STRING_BOXED_OBJECT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* STRING:OBJECT (./glade-marshallers.list:20) */
+/* STRING:OBJECT (./glade-marshallers.list:22) */
 extern void glade_marshal_STRING__OBJECT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -889,7 +935,7 @@ glade_marshal_STRING__OBJECT (GClosure     *closure,
   g_value_take_string (return_value, v_return);
 }
 
-/* INT:OBJECT,BOXED (./glade-marshallers.list:21) */
+/* INT:OBJECT,BOXED (./glade-marshallers.list:23) */
 extern void glade_marshal_INT__OBJECT_BOXED (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
