@@ -58,8 +58,6 @@
 #define GLADE_UTIL_SELECTION_NODE_SIZE 7
 #define GLADE_UTIL_COPY_BUFFSIZE       1024
 
-#define GLADE_DEVHELP_ICON_NAME           "devhelp"
-#define GLADE_DEVHELP_FALLBACK_ICON_FILE  "devhelp.png"
 
 /* List of widgets that have selection
  */
@@ -174,7 +172,7 @@ glade_utils_get_pspec_from_funcname (const gchar *funcname)
 			      (gpointer) &get_pspec)) {
 		g_warning (_("We could not find the symbol \"%s\""),
 			   funcname);
-		return FALSE;
+		return NULL;
 	}
 
 	g_assert (get_pspec);
@@ -186,8 +184,9 @@ glade_utils_get_pspec_from_funcname (const gchar *funcname)
 /**
  * glade_util_ui_message:
  * @parent: a #GtkWindow cast as a #GtkWidget
- * @format: a printf style format string
  * @type:   a #GladeUIMessageType
+ * @widget: a #GtkWidget to append to the dialog vbox
+ * @format: a printf style format string
  * @...:    args for the format.
  *
  * Creates a new warning dialog window as a child of @parent containing
@@ -198,9 +197,10 @@ glade_utils_get_pspec_from_funcname (const gchar *funcname)
  *          selected "OK", True if the @type was GLADE_UI_YES_OR_NO and
  *          the user selected "YES"; False otherwise.
  */
-gboolean
+gint
 glade_util_ui_message (GtkWidget           *parent, 
 		       GladeUIMessageType   type,
+		       GtkWidget           *widget,
 		       const gchar         *format,
 		       ...)
 {
@@ -252,8 +252,15 @@ glade_util_ui_message (GtkWidget           *parent,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 message_type,
 					 buttons_type,
+					 "%s",
 					 string);
 
+	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
+
+	if (widget)
+		gtk_box_pack_end (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+				  widget, TRUE, TRUE, 2);
+
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
 	gtk_widget_destroy (dialog);
@@ -263,8 +270,28 @@ glade_util_ui_message (GtkWidget           *parent,
 }
 
 
+gboolean
+glade_util_check_and_warn_scrollable (GladeWidget        *parent,
+				      GladeWidgetAdaptor *child_adaptor,
+				      GtkWidget          *parent_widget)
+{
+	if (GTK_IS_SCROLLED_WINDOW (parent->object) &&
+	    GWA_SCROLLABLE_WIDGET (child_adaptor) == FALSE)
+	{
+		GladeWidgetAdaptor *vadaptor = 
+			glade_widget_adaptor_get_by_type (GTK_TYPE_VIEWPORT);
 
-
+		glade_util_ui_message (parent_widget,
+				       GLADE_UI_INFO, NULL,
+				       _("Cannot add non scrollable %s widget to a %s directly.\n"
+					 "Add a %s first."),
+				       child_adaptor->title,
+				       parent->adaptor->title,
+				       vadaptor->title);
+		return TRUE;
+	}
+	return FALSE;
+}
 
 typedef struct {
 	GtkStatusbar *statusbar;
@@ -272,7 +299,7 @@ typedef struct {
 	guint message_id;
 } FlashInfo;
 
-static const guint32 flash_length = 3000;
+static const guint flash_length = 3;
 
 static gboolean
 remove_message_timeout (FlashInfo * fi) 
@@ -311,7 +338,7 @@ glade_util_flash_message (GtkWidget *statusbar, guint context_id, gchar *format,
 	fi->context_id = context_id;	
 	fi->message_id = gtk_statusbar_push (fi->statusbar, fi->context_id, message);
 
-	gtk_timeout_add (flash_length, (GtkFunction) remove_message_timeout, fi);
+	g_timeout_add_seconds (flash_length, (GtkFunction) remove_message_timeout, fi);
 
 	g_free (message);
 }
@@ -412,10 +439,10 @@ glade_util_gtk_combo_func (gpointer data)
 	GtkWidget *label;
 	gchar *ltext = NULL;
 
-	ltext = (gchar *) gtk_object_get_data (GTK_OBJECT (listitem),
-					       gtk_combo_string_key);
+	ltext = (gchar *) g_object_get_data (G_OBJECT (listitem),
+					     gtk_combo_string_key);
 	if (!ltext) {
-		label = GTK_BIN (listitem)->child;
+		label = gtk_bin_get_child (GTK_BIN (listitem));
 		if (!label || !GTK_IS_LABEL (label))
 			return NULL;
 		ltext = (gchar*) gtk_label_get_text (GTK_LABEL (label));
@@ -439,21 +466,24 @@ glade_util_gtk_combo_find (GtkCombo * combo)
 	gchar *text;
 	gchar *ltext;
 	GList *clist;
-	int (*string_compare) (const char *, const char *);
+	gsize len;
+
+	int (*string_compare) (const char *, const char *, gsize);
 
 	if (combo->case_sensitive)
-		string_compare = strcmp;
+		string_compare = strncmp;
 	else
-		string_compare = g_strcasecmp;
+		string_compare = g_ascii_strncasecmp;
 
-	text = (gchar*) gtk_entry_get_text (GTK_ENTRY (combo->entry));
+	text  = (gchar*) gtk_entry_get_text (GTK_ENTRY (combo->entry));
+	len   = text ? strlen (text) : 0;
 	clist = GTK_LIST (combo->list)->children;
 
 	while (clist && clist->data) {
 		ltext = glade_util_gtk_combo_func (GTK_LIST_ITEM (clist->data));
 		if (!ltext)
 			continue;
-		if (!(*string_compare) (ltext, text))
+		if (!(*string_compare) (ltext, text, len))
 			return (GtkListItem *) clist->data;
 		clist = clist->next;
 	}
@@ -506,7 +536,7 @@ add_format_options (GtkDialog    *dialog,
 	GtkWidget *vbox, *frame;
 	GtkWidget *glade_radio, *builder_radio;
 	GtkWidget *label, *alignment;
-	gchar     *string = g_strdup_printf ("<b>%s</b>", _("Select file format"));
+	gchar     *string = g_strdup_printf ("<b>%s</b>", _("File format"));
 
 	frame = gtk_frame_new (NULL);
 	vbox = gtk_vbox_new (FALSE, 0);
@@ -544,7 +574,7 @@ add_format_options (GtkDialog    *dialog,
 
 	gtk_widget_show_all (frame);
 	
-	gtk_box_pack_end (GTK_BOX (dialog->vbox), frame, FALSE, TRUE, 2);
+	gtk_box_pack_end (GTK_BOX (gtk_dialog_get_content_area (dialog)), frame, FALSE, TRUE, 2);
 }
 
 
@@ -700,7 +730,7 @@ glade_util_get_window_positioned_in (GtkWidget *widget)
 {
 	GtkWidget *parent;
 
-	parent = widget->parent;
+	parent = gtk_widget_get_parent (widget);
 
 #ifdef USE_GNOME
 	/* BonoboDockItem widgets use a different window when floating.
@@ -718,9 +748,9 @@ glade_util_get_window_positioned_in (GtkWidget *widget)
 #endif
 
 	if (parent)
-		return parent->window;
+		return gtk_widget_get_window (parent);
 
-	return widget->window;
+	return gtk_widget_get_window (widget);
 }
 
 static void
@@ -789,10 +819,10 @@ glade_util_can_draw_nodes (GtkWidget *sel_widget, GdkWindow *sel_win,
 	GdkWindow *viewport_win = NULL;
 
 	/* Check if the selected widget is inside a viewport. */
-	for (widget = sel_widget->parent; widget; widget = widget->parent) {
+	for (widget = gtk_widget_get_parent (sel_widget); widget; widget = gtk_widget_get_parent (widget)) {
 		if (GTK_IS_VIEWPORT (widget)) {
 			viewport = widget;
-			viewport_win = GTK_VIEWPORT (widget)->bin_window;
+			viewport_win = gtk_viewport_get_bin_window (GTK_VIEWPORT (widget));
 			break;
 		}
 	}
@@ -842,7 +872,7 @@ glade_util_draw_selection_nodes (GdkWindow *expose_win)
 	/* Find the corresponding GtkWidget */
 	gdk_window_get_user_data (expose_win, (gpointer)&expose_widget);
 
-	gc = expose_widget->style->black_gc;
+	gc = gtk_widget_get_style (expose_widget)->black_gc;
 
 	/* Calculate the offset of the expose window within its toplevel. */
 	glade_util_calculate_window_offset (expose_win,
@@ -875,10 +905,13 @@ glade_util_draw_selection_nodes (GdkWindow *expose_win)
 		if (expose_toplevel == sel_toplevel
 		    && glade_util_can_draw_nodes (sel_widget, sel_win,
 						  expose_win)) {
-			x = sel_x + sel_widget->allocation.x - expose_win_x;
-			y = sel_y + sel_widget->allocation.y - expose_win_y;
-			w = sel_widget->allocation.width;
-			h = sel_widget->allocation.height;
+			GtkAllocation allocation;
+
+			gtk_widget_get_allocation (sel_widget, &allocation);
+			x = sel_x + allocation.x - expose_win_x;
+			y = sel_y + allocation.y - expose_win_y;
+			w = allocation.width;
+			h = allocation.height;
 
 			/* Draw the selection nodes if they intersect the
 			   expose window bounds. */
@@ -918,6 +951,8 @@ glade_util_add_selection (GtkWidget *widget)
 void
 glade_util_remove_selection (GtkWidget *widget)
 {
+	GtkWidget *parent;
+
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 	if (!glade_util_has_selection (widget))
 		return;
@@ -927,8 +962,8 @@ glade_util_remove_selection (GtkWidget *widget)
 
 	/* We redraw the parent, since the selection rectangle may not be
 	   cleared if we just redraw the widget itself. */
-	gtk_widget_queue_draw (widget->parent ?
-			       widget->parent : widget);
+	parent = gtk_widget_get_parent (widget);
+	gtk_widget_queue_draw (parent ? parent : widget);
 }
 
 /**
@@ -940,6 +975,7 @@ void
 glade_util_clear_selection (void)
 {
 	GtkWidget *widget;
+	GtkWidget *parent;
 	GList     *list;
 
 	for (list = glade_util_selection;
@@ -947,8 +983,8 @@ glade_util_clear_selection (void)
 	     list = list->next)
 	{
 		widget = list->data;
-		gtk_widget_queue_draw (widget->parent ?
-				       widget->parent : widget);
+		parent = gtk_widget_get_parent (widget);
+		gtk_widget_queue_draw (parent ? parent : widget);
 	}
 	glade_util_selection =
 		(g_list_free (glade_util_selection), NULL);
@@ -1054,7 +1090,7 @@ glade_util_find_iter (GtkTreeModel *model,
 		      gint          column)
 {
 	GtkTreeIter *retval = NULL;
-	GladeWidget *widget;
+	GObject* object;
 	GtkTreeIter *next;
 
 	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), NULL);
@@ -1065,12 +1101,8 @@ glade_util_find_iter (GtkTreeModel *model,
 
 	while (retval == NULL)
 	{
-		gtk_tree_model_get (model, next, column, &widget, -1);
-		if (widget == NULL) {
-			g_warning ("Could not get the glade widget from the model");
-			break;
-		}
-		else if (widget == findme)
+		gtk_tree_model_get (model, next, column, &object, -1);
+		if (object == glade_widget_get_object (findme))
 		{
 			retval = gtk_tree_iter_copy (next);
 			break;
@@ -1084,6 +1116,8 @@ glade_util_find_iter (GtkTreeModel *model,
 				break;
 		}
 
+		g_object_unref (object);
+		
 		if (!gtk_tree_model_iter_next (model, next))
 			break;
 	}
@@ -1308,7 +1342,7 @@ glade_util_copy_file (const gchar  *src_path,
 
 	if (g_file_test (dest_path, G_FILE_TEST_IS_REGULAR) != FALSE)
 		if (glade_util_ui_message
-		    (glade_app_get_window(), GLADE_UI_YES_OR_NO,
+		    (glade_app_get_window(), GLADE_UI_YES_OR_NO, NULL,
 		     _("%s exists.\nDo you want to replace it?"), dest_path) == FALSE)
 		    return FALSE;
 
@@ -1336,7 +1370,7 @@ glade_util_copy_file (const gchar  *src_path,
 				if (write_status == G_IO_STATUS_ERROR)
 				{
 					glade_util_ui_message (glade_app_get_window(),
-							       GLADE_UI_ERROR,
+							       GLADE_UI_ERROR, NULL,
 							       _("Error writing to %s: %s"),
 							       dest_path, error->message);
 					error = (g_error_free (error), NULL);
@@ -1350,7 +1384,7 @@ glade_util_copy_file (const gchar  *src_path,
 			if (read_status == G_IO_STATUS_ERROR)
 			{
 				glade_util_ui_message (glade_app_get_window(),
-						       GLADE_UI_ERROR,
+						       GLADE_UI_ERROR, NULL,
 						       _("Error reading %s: %s"),
 						       src_path, error->message);
 				error = (g_error_free (error), NULL);
@@ -1365,7 +1399,7 @@ glade_util_copy_file (const gchar  *src_path,
 			{
 				glade_util_ui_message
 					(glade_app_get_window(),
-					 GLADE_UI_ERROR,
+					 GLADE_UI_ERROR, NULL,
 					 _("Error shutting down I/O channel %s: %s"),
 						       dest_path, error->message);
 				error = (g_error_free (error), NULL);
@@ -1375,7 +1409,7 @@ glade_util_copy_file (const gchar  *src_path,
 		else
 		{
 			glade_util_ui_message (glade_app_get_window(),
-					       GLADE_UI_ERROR,
+					       GLADE_UI_ERROR, NULL,
 					       _("Failed to open %s for writing: %s"), 
 					       dest_path, error->message);
 			error = (g_error_free (error), NULL);
@@ -1386,8 +1420,8 @@ glade_util_copy_file (const gchar  *src_path,
 		if (g_io_channel_shutdown (src, TRUE, &error) != G_IO_STATUS_NORMAL)
 		{
 			glade_util_ui_message (glade_app_get_window(),
-					       GLADE_UI_ERROR,
-					       _("Error shutting down io channel %s: %s"),
+					       GLADE_UI_ERROR, NULL,
+					       _("Error shutting down I/O channel %s: %s"),
 					       src_path, error->message);
 			success = FALSE;
 		}
@@ -1395,7 +1429,7 @@ glade_util_copy_file (const gchar  *src_path,
 	else 
 	{
 		glade_util_ui_message (glade_app_get_window(),
-				       GLADE_UI_ERROR,
+				       GLADE_UI_ERROR, NULL,
 				       _("Failed to open %s for reading: %s"), 
 				       src_path, error->message);
 		error = (g_error_free (error), NULL);
@@ -1431,16 +1465,19 @@ glade_util_class_implements_interface (GType class_type,
 	return implemented;
 }
 
-
 static GModule *
 try_load_library (const gchar *library_path,
 		  const gchar *library_name)
 {
-	GModule *module;
+	GModule *module = NULL;
 	gchar   *path;
 
 	path = g_module_build_path (library_path, library_name);
-	module = g_module_open (path, G_MODULE_BIND_LAZY);
+	if (g_file_test (path, G_FILE_TEST_EXISTS))
+	{
+		if (!(module = g_module_open (path, G_MODULE_BIND_LAZY)))
+			g_warning ("Failed to load %s: %s", path, g_module_error ());
+	}
 	g_free (path);
 
 	return module;
@@ -1461,12 +1498,20 @@ try_load_library (const gchar *library_path,
 GModule *
 glade_util_load_library (const gchar *library_name)
 {
+	gchar        *default_paths[] = { (gchar *)glade_app_get_modules_dir (), 
+					  NULL, /* <-- dynamically allocated */ 
+					  "/lib", 
+					  "/usr/lib", 
+					  "/usr/local/lib", 
+					  NULL };
+
 	GModule      *module = NULL;
-	const gchar  *default_paths[] = { glade_app_get_modules_dir (), "/lib", "/usr/lib", "/usr/local/lib", NULL };
 	const gchar  *search_path;
 	gchar       **split;
 	gint          i;
+
 	
+
 	if ((search_path = g_getenv (GLADE_ENV_MODULE_PATH)) != NULL)
 	{
 		if ((split = g_strsplit (search_path, ":", 0)) != NULL)
@@ -1481,9 +1526,14 @@ glade_util_load_library (const gchar *library_name)
 
 	if (!module)
 	{
+		/* Search ${prefix}/lib after searching ${prefix}/lib/glade3/modules... */
+		default_paths[1] = g_build_filename (glade_app_get_modules_dir (), "..", "..", NULL);
+
 		for (i = 0; default_paths[i] != NULL; i++)
 			if ((module = try_load_library (default_paths[i], library_name)) != NULL)
 				break;
+
+		g_free (default_paths[1]);
 	}
 
 	if (!module)
@@ -1638,18 +1688,19 @@ glade_util_search_devhelp (const gchar *book,
 			   const gchar *search)
 {
 	GError *error = NULL;
-	gchar  *book_comm = NULL, *page_comm = NULL;
+	gchar  *book_comm = NULL, *page_comm = NULL, *search_comm = NULL;
 	gchar  *string;
 
 	g_return_if_fail (glade_util_have_devhelp ());
 
-	if (book) book_comm = g_strdup_printf ("book:%s ", book);
-	if (page) page_comm = g_strdup_printf ("page:%s ", page);
+	if (book) book_comm = g_strdup_printf ("book:%s", book);
+	if (page) page_comm = g_strdup_printf (" page:%s", page);
+	if (search) search_comm = g_strdup_printf (" %s", search);
 
 	string = g_strdup_printf ("devhelp -s \"%s%s%s\"", 
 				   book_comm ? book_comm : "",
 				   page_comm ? page_comm : "",
-				   search ? search : "");
+				   search_comm ? search_comm : "");
 
 	if (g_spawn_command_line_async (string, &error) == FALSE)
 	{
@@ -1660,6 +1711,7 @@ glade_util_search_devhelp (const gchar *book,
 	g_free (string);
 	if (book_comm) g_free (book_comm);
 	if (page_comm) g_free (page_comm);
+	if (search_comm) g_free (search_comm);
 }
 
 GtkWidget *
@@ -1667,6 +1719,7 @@ glade_util_get_placeholder_from_pointer (GtkContainer *container)
 {
 	GtkWidget *toplevel;
 	GtkWidget *retval = NULL, *child;
+	GtkAllocation allocation;
 	GList *c, *l;
 	gint x, y, x2, y2;
 
@@ -1683,15 +1736,15 @@ glade_util_get_placeholder_from_pointer (GtkContainer *container)
 		child = l->data;
 		
 		if (GLADE_IS_PLACEHOLDER (child) &&
-		    GTK_WIDGET_MAPPED (child))
+		    gtk_widget_get_mapped (child))
 		{
 			gtk_widget_translate_coordinates (toplevel, child,
 							  x, y, &x2, &y2);
 
-			
+			gtk_widget_get_allocation (child, &allocation);
 			if (x2 >= 0 && y2 >= 0 &&
-			    x2 <= child->allocation.width &&
-			    y2 <= child->allocation.height)
+			    x2 <= allocation.width &&
+			    y2 <= allocation.height)
 			{
 				retval = child;
 				break;
@@ -1897,3 +1950,447 @@ glade_util_get_file_mtime (const gchar *filename, GError **error)
 		return info.st_mtime;
 	}
 }
+
+gchar *
+glade_util_filename_to_icon_name (const gchar *value)
+{
+	gchar *icon_name, *p;
+	g_return_val_if_fail (value && value[0], NULL);
+
+	icon_name = g_strdup_printf ("glade-generated-%s", value);
+
+	if ((p = strrchr (icon_name, '.')) != NULL)
+		*p = '-';
+	
+	return icon_name;
+}
+
+gchar *
+glade_util_icon_name_to_filename (const gchar *value)
+{
+	/* sscanf makes us allocate a buffer */
+	gchar filename[FILENAME_MAX], *p;
+	g_return_val_if_fail (value && value[0], NULL);
+
+	sscanf (value, "glade-generated-%s", filename);
+
+	/* XXX: Filenames without an extention will evidently
+	 * break here
+	 */
+	if ((p = strrchr (filename, '-')) != NULL)
+		*p = '.';
+	
+	return g_strdup (filename);
+}
+
+gint
+glade_utils_enum_value_from_string (GType enum_type, const gchar *strval)
+{
+	gint          value = 0;
+	const gchar  *displayable;
+	GValue       *gvalue;
+
+	g_return_val_if_fail (strval && strval[0], 0);
+
+	if (((displayable = glade_get_value_from_displayable (enum_type, strval)) != NULL &&
+	     (gvalue = glade_utils_value_from_string (enum_type, displayable, NULL, NULL)) != NULL) ||
+	    (gvalue = glade_utils_value_from_string (enum_type, strval, NULL, NULL)) != NULL)
+	{
+		value = g_value_get_enum (gvalue);
+		g_value_unset (gvalue);
+		g_free (gvalue);
+	}
+	return value;
+}
+
+static gchar *
+glade_utils_enum_string_from_value_real (GType enum_type, gint value, gboolean displayable)
+{
+	GValue gvalue = { 0, };
+	gchar *string;
+
+	g_value_init (&gvalue, enum_type);
+	g_value_set_enum (&gvalue, value);
+
+	string = glade_utils_string_from_value (&gvalue, GLADE_PROJECT_FORMAT_GTKBUILDER);
+	g_value_unset (&gvalue);
+
+	if (displayable && string)
+	{
+		const gchar *dstring = glade_get_displayable_value (enum_type, string);
+		if (dstring)
+		{
+			g_free (string);
+			return g_strdup (dstring);
+		}
+	}
+
+	return string;
+}
+
+gchar *
+glade_utils_enum_string_from_value (GType enum_type, gint value)
+{
+	return glade_utils_enum_string_from_value_real (enum_type, value, FALSE);
+}
+
+gchar *
+glade_utils_enum_string_from_value_displayable (GType enum_type, gint value)
+{
+	return glade_utils_enum_string_from_value_real (enum_type, value, TRUE);
+}
+
+
+gint
+glade_utils_flags_value_from_string (GType flags_type, const gchar *strval)
+{
+	gint          value = 0;
+	const gchar  *displayable;
+	GValue       *gvalue;
+
+	g_return_val_if_fail (strval && strval[0], 0);
+
+	if (((displayable = glade_get_value_from_displayable (flags_type, strval)) != NULL &&
+	     (gvalue = glade_utils_value_from_string (flags_type, displayable, NULL, NULL)) != NULL) ||
+	    (gvalue = glade_utils_value_from_string (flags_type, strval, NULL, NULL)) != NULL)
+	{
+		value = g_value_get_flags (gvalue);
+		g_value_unset (gvalue);
+		g_free (gvalue);
+	}
+	return value;
+}
+
+static gchar *
+glade_utils_flags_string_from_value_real (GType flags_type, gint value, gboolean displayable)
+{
+	GValue gvalue = { 0, };
+	gchar *string;
+
+	g_value_init (&gvalue, flags_type);
+	g_value_set_flags (&gvalue, value);
+
+	string = glade_utils_string_from_value (&gvalue, GLADE_PROJECT_FORMAT_GTKBUILDER);
+	g_value_unset (&gvalue);
+
+	if (displayable && string)
+	{
+		const gchar *dstring = glade_get_displayable_value (flags_type, string);
+		if (dstring)
+		{
+			g_free (string);
+			return g_strdup (dstring);
+		}
+	}
+
+	return string;
+}
+
+gchar *
+glade_utils_flags_string_from_value (GType flags_type, gint value)
+{
+	return glade_utils_flags_string_from_value_real (flags_type, value, FALSE);
+
+}
+
+
+gchar *
+glade_utils_flags_string_from_value_displayable (GType flags_type, gint value)
+{
+	return glade_utils_flags_string_from_value_real (flags_type, value, TRUE);
+}
+
+
+/* A hash table of generically created property classes for
+ * fundamental types, so we can easily use glade's conversion
+ * system without using properties (only GTypes)
+ */
+static GHashTable *generic_property_classes = NULL;
+
+
+static gboolean
+utils_gtype_equal (gconstpointer v1,
+		 gconstpointer v2)
+{
+  return *((const GType*) v1) == *((const GType*) v2);
+}
+
+static guint
+utils_gtype_hash (gconstpointer v)
+{
+  return *(const GType*) v;
+}
+
+
+static GladePropertyClass *
+pclass_from_gtype (GType type)
+{
+	GladePropertyClass *property_class = NULL;
+	GParamSpec         *pspec = NULL;
+
+	if (!generic_property_classes)
+		generic_property_classes = g_hash_table_new_full (utils_gtype_hash, utils_gtype_equal,
+								  g_free, (GDestroyNotify)glade_property_class_free);
+	
+	property_class = g_hash_table_lookup (generic_property_classes, &type);
+
+	if (!property_class)
+	{
+		/* Support enum and flag types, and a hardcoded list of fundamental types */
+		if (type == G_TYPE_CHAR)
+			pspec = g_param_spec_char ("dummy", "dummy", "dummy",
+						   G_MININT8, G_MAXINT8, 0, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_UCHAR)
+			pspec = g_param_spec_char ("dummy", "dummy", "dummy",
+						   0, G_MAXUINT8, 0, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_BOOLEAN)
+			pspec = g_param_spec_boolean ("dummy", "dummy", "dummy",
+						      FALSE, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_INT)
+			pspec = g_param_spec_int ("dummy", "dummy", "dummy",
+						  G_MININT, G_MAXINT, 0, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_UINT)
+			pspec = g_param_spec_uint ("dummy", "dummy", "dummy",
+						   0, G_MAXUINT, 0, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_LONG)
+			pspec = g_param_spec_long ("dummy", "dummy", "dummy",
+						    G_MINLONG, G_MAXLONG, 0, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_ULONG)
+			pspec = g_param_spec_ulong ("dummy", "dummy", "dummy",
+						    0, G_MAXULONG, 0, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_INT64)
+			pspec = g_param_spec_int64 ("dummy", "dummy", "dummy",
+						    G_MININT64, G_MAXINT64, 0, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_UINT64)
+			pspec = g_param_spec_uint64 ("dummy", "dummy", "dummy",
+						     0, G_MAXUINT64, 0, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_FLOAT)
+			pspec = g_param_spec_float ("dummy", "dummy", "dummy",
+						    G_MINFLOAT, G_MAXFLOAT, 1.0F, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_DOUBLE)
+			pspec = g_param_spec_double ("dummy", "dummy", "dummy",
+						     G_MINDOUBLE, G_MAXDOUBLE, 1.0F, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_STRING)
+			pspec = g_param_spec_string ("dummy", "dummy", "dummy",
+						     NULL, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (type == G_TYPE_OBJECT || g_type_is_a (type, G_TYPE_OBJECT))
+			pspec = g_param_spec_object ("dummy", "dummy", "dummy",
+						     type, G_PARAM_READABLE|G_PARAM_WRITABLE);
+		else if (G_TYPE_IS_ENUM (type))
+		{
+			GEnumClass *eclass = g_type_class_ref (type);
+			pspec = g_param_spec_enum ("dummy", "dummy", "dummy",
+						   type, eclass->minimum, G_PARAM_READABLE|G_PARAM_WRITABLE);
+			g_type_class_unref (eclass);
+		}
+		else if (G_TYPE_IS_FLAGS (type))
+			pspec = g_param_spec_flags ("dummy", "dummy", "dummy",
+						    type, 0, G_PARAM_READABLE|G_PARAM_WRITABLE);
+
+		if (pspec)
+		{
+			if ((property_class = 
+			     glade_property_class_new_from_spec_full (NULL, pspec, FALSE)) != NULL)
+			{
+				/* XXX If we ever free the hash table, property classes wont touch
+				 * the allocated pspecs, so they would theoretically be leaked.
+				 */
+				g_hash_table_insert (generic_property_classes, 
+						     g_memdup (&type, sizeof (GType)), property_class);
+			}
+			else
+				g_warning ("Unable to create property class for type %s", g_type_name (type));
+		}
+		else
+			g_warning ("No generic conversion support for type %s", g_type_name (type));
+	}
+	return property_class;
+}
+
+/**
+ * glade_utils_value_from_string:
+ * @type: a #GType to convert with
+ * @string: the string to convert
+ * @project: the #GladeProject to look for formats of object names when needed
+ * @widget: if the value is a gobject, this #GladeWidget will be used to look
+ *          for an object in the same widget tree.
+ *
+ * Allocates and sets a #GValue of type @type
+ * set to @string (using glade conversion routines) 
+ *
+ * Returns: A newly allocated and set #GValue
+ */
+GValue *
+glade_utils_value_from_string (GType               type,
+			       const gchar        *string,
+			       GladeProject       *project,
+			       GladeWidget        *widget)
+{
+	GladePropertyClass *pclass;
+
+	g_return_val_if_fail (type != G_TYPE_INVALID, NULL);
+	g_return_val_if_fail (string != NULL, NULL);
+
+	if ((pclass = pclass_from_gtype (type)) != NULL)
+		return glade_property_class_make_gvalue_from_string (pclass, string, project, widget);
+
+	return NULL;
+}
+
+
+/**
+ * glade_utils_string_from_value:
+ * @value: a #GValue to convert
+ * @fmt: the #GladeProjectFormat to honor
+ *
+ * Serializes #GValue into a string 
+ * (using glade conversion routines) 
+ *
+ * Returns: A newly allocated string
+ */
+gchar *
+glade_utils_string_from_value (const GValue       *value,
+			       GladeProjectFormat  fmt)
+{
+	GladePropertyClass *pclass;
+
+	g_return_val_if_fail (value != NULL, NULL);
+
+	if ((pclass = pclass_from_gtype (G_VALUE_TYPE (value))) != NULL)
+		return glade_property_class_make_string_from_gvalue (pclass, value, fmt);
+
+	return NULL;
+}
+
+
+/**
+ * glade_utils_liststore_from_enum_type:
+ * @enum_type: A #GType
+ * @include_empty: wheather to prepend an "Unset" slot
+ *
+ * Creates a liststore suitable for comboboxes and such to 
+ * chose from a variety of types.
+ *
+ * Returns: A new #GtkListStore
+ */
+GtkListStore *
+glade_utils_liststore_from_enum_type (GType    enum_type,
+				      gboolean include_empty)
+{
+	GtkListStore        *store;
+	GtkTreeIter          iter;
+	GEnumClass          *eclass;
+	guint                i;
+
+	eclass = g_type_class_ref (enum_type);
+
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+
+	if (include_empty)
+	{
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 
+				    0, _("None"),
+				    -1);
+	}
+	
+	for (i = 0; i < eclass->n_values; i++)
+	{
+		const gchar *displayable = glade_get_displayable_value (enum_type, eclass->values[i].value_nick);
+
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 
+				    0, displayable ? displayable : eclass->values[i].value_nick,
+				    -1);
+	}
+
+	g_type_class_unref (eclass);
+
+	return store;
+}
+
+
+
+/**
+ * glade_utils_hijack_key_press:
+ * @win: a #GtkWindow
+ * event: the GdkEventKey 
+ * user_data: unused
+ *
+ * This function is meant to be attached to key-press-event of a toplevel,
+ * it simply allows the window contents to treat key events /before/ 
+ * accelerator keys come into play (this way widgets dont get deleted
+ * when cutting text in an entry etc.).
+ * Creates a liststore suitable for comboboxes and such to 
+ * chose from a variety of types.
+ *
+ * Returns: whether the event was handled
+ */
+gint
+glade_utils_hijack_key_press (GtkWindow          *win, 
+			      GdkEventKey        *event, 
+			      gpointer            user_data)
+{
+	GtkWidget *focus_widget;
+
+	focus_widget = gtk_window_get_focus (win);
+	if (focus_widget &&
+	    (event->keyval == GDK_Delete || /* Filter Delete from accelerator keys */
+	     ((event->state & GDK_CONTROL_MASK) && /* CNTL keys... */
+	      ((event->keyval == GDK_c || event->keyval == GDK_C) || /* CNTL-C (copy)  */
+	       (event->keyval == GDK_x || event->keyval == GDK_X) || /* CNTL-X (cut)   */
+	       (event->keyval == GDK_v || event->keyval == GDK_V) || /* CNTL-V (paste) */
+	       (event->keyval == GDK_n || event->keyval == GDK_N))))) /* CNTL-N (new project) */
+	{
+		return gtk_widget_event (focus_widget,
+					 (GdkEvent *)event);
+	}
+	return FALSE;
+}
+
+
+/* copied from gedit */
+gchar *
+glade_utils_replace_home_dir_with_tilde (const gchar *uri)
+{
+	gchar *tmp;
+	gchar *home;
+
+	g_return_val_if_fail (uri != NULL, NULL);
+
+	/* Note that g_get_home_dir returns a const string */
+	tmp = (gchar *)g_get_home_dir ();
+
+	if (tmp == NULL)
+		return g_strdup (uri);
+
+	home = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
+	if (home == NULL)
+		return g_strdup (uri);
+
+	if (strcmp (uri, home) == 0)
+	{
+		g_free (home);
+		
+		return g_strdup ("~");
+	}
+
+	tmp = home;
+	home = g_strdup_printf ("%s/", tmp);
+	g_free (tmp);
+
+	if (g_str_has_prefix (uri, home))
+	{
+		gchar *res;
+
+		res = g_strdup_printf ("~/%s", uri + strlen (home));
+
+		g_free (home);
+		
+		return res;		
+	}
+
+	g_free (home);
+
+	return g_strdup (uri);
+}
