@@ -72,6 +72,7 @@ typedef enum {
 #define GLADE_XML_TAG_SIGNAL                      "signal"
 #define GLADE_XML_TAG_HANDLER                     "handler"
 #define GLADE_XML_TAG_AFTER                       "after"
+#define GLADE_XML_TAG_SWAPPED                     "swapped"
 #define GLADE_XML_TAG_OBJECT                      "object"
 #define GLADE_XML_TAG_NAME                        "name"
 #define GLADE_XML_TAG_CHILD                       "child"
@@ -80,6 +81,7 @@ typedef enum {
 #define GLADE_XML_TAG_INTERNAL_CHILD              "internal-child"
 #define GLADE_XML_TAG_I18N_TRUE                   "yes"
 #define GLADE_XML_TAG_SIGNAL_TRUE                 "yes"
+#define GLADE_XML_TAG_SIGNAL_FALSE                "no"
 #define GLADE_XML_TAG_TYPE                        "type"
 #define GLADE_XML_TAG_SOURCES                     "sources"
 #define GLADE_XML_TAG_SOURCE                      "source"
